# TCPquery.pm

# ===========================================================================
# file         : TCPquery.pm
# copyright    : 2005, 2006 by Sven 'ScP' Paulsen & Felix 'WeeZer' Siefert
# email        : scp@planetteamspeak.com & weezer@planetteamspeak.com
# version      : 0.7.2
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 675 Mass Ave, Cambridge, MA 02139, USA.
# ===========================================================================

package TeamSpeak2::TCPquery;

use TeamSpeak2::TCPquery::Server;
use TeamSpeak2::TCPquery::Player;
use TeamSpeak2::TCPquery::Channel;

use strict;
use warnings;
use diagnostics;

use IO::Socket;
use Data::Dumper;

use constant PACKAGE => {
  "name"      => "TeamSpeak2::TCPquery",
  "version"   => "0.7.2",
  "debugging" => 1,
};

use constant PROMPT  => "[TS]";
use constant SUCCESS => "OK";

use constant ERROR => {
  "weird"        => "ERROR, error",
  "id"           => "ERROR, invalid id",
  "data"         => "ERROR, no data available",
  "player"       => "ERROR, Playerid does not exist",
  "channel"      => "ERROR, Channelid does not exist",
  "incomplete"   => "ERROR, invalid paramcount",
  "password"     => "ERROR, password dont match",
  "passwort"     => "ERROR, passwort dont match",
  "format"       => "ERROR, invalid number format",
  "unauthorized" => "ERROR, not logged in",
  "select"       => "ERROR, no server selected",
  "login"        => "ERROR, invalid login",
  "blocked"      => "Your password failed 3 consecutive times, please wait 10 minutes before trying again!",
  "banned"       => "You are still banned or a failed password try caused a 30 second delay!",
  "unknown"      => "error",
};

use constant TITLE => {
  "pl"           => "p_id\tc_id\tps\tbs\tpr\tbr\tpl\tping\tlogintime\tidletime\tcprivs\tpprivs\tpflags\tip\tnick\tloginname",
  "cl"           => "id\tcodec\tparent\torder\tmaxusers\tname\tflags\tpassword\ttopic",
  "fc"           => "c_id\tc_dbid\tc_name",
  "fp"           => "p_id\tp_dbid\tc_id\tnickname\tloginname\tip", 
  "ci_1"         => "c_id\tc_pid\tc_dbid\tc_name\tc_fU\tc_fM\tc_fP\tc_fH\tc_fD\tc_codec\tc_order\tc_maxusers\tc_created\tc_topic\tc_description\t",
  "ci_2"         => "p_id\tp_nick\tsa\tca\to\tao\tv\tav\tcst\treg",
  "dbci_1"       => "c_dbid\tc_dbpid\tc_name\tc_fU\tc_fM\tc_fP\tc_fH\tc_fD\tc_codec\tc_order\tc_maxusers\tc_created\tc_topic\tc_description",
  "dbci_2"       => "p_dbid\tloginname\tca\tao\tav",
  "dbfp"         => "p_dbid\tloginname\tsa\tcreated\tlastonline",
  "pi_1"         => "p_id\tp_dbid\tc_id\tnickname\tloginname\tsa\tst\tip",
  "pi_2"         => "c_id\tname\tca\to\tao\tv\tav",
  "dbpi_1"       => "p_dbid\tloginname\tsa\tcreated\tlastonline",
  "dbpi_2"       => "c_dbid\tc_dbpid\tc_name\tca\tao\tav",
  "dccl"         => "c_id\tc_name\tp_id\tp_name",
  "dbserverlist" => "id\tudpport\tname\tstatus",
  "dbuserlist"   => "id\tsa\tcreated\tlastonline\tname",
  "dbsuserlist"  => "id\tcreated\tlastonline\tname",
  "lc"           => "login_name\tserver_id\tssa\tsa\tip\tlast_command",
  "banlist"      => "b_id\tip\tcreated\tmins\tby",
};

my $ini_date_format = '\d\d-\d\d-\d\d\d\d\s\d\d:\d\d:\d\d';

my @debug= ();
my (@playerlist, @channellist, @serverlist, @dbserverlist) = ();

# ===========================================================================
# There should be no need to change anything below
# ===========================================================================

# Class method: new()
# Creates a new TeamSpeak2::TCPquery object and connects to a TeamSpeak 2
# servers TCP query interface by using the IO::Socket::INET package
#
# EXAMPLE:
#   my $obj = TeamSpeak2::TCPquery->new("localhost","51234");
#
sub new {
  my ($class, $server_address, $server_tcpport) = @_;
  my $socket = IO::Socket::INET->new(PeerAddr => $server_address, PeerPort => $server_tcpport, Proto => "tcp") || return 0;
  $socket->autoflush(1);
  $socket->timeout(1);
  my $self = bless {
    handle   => $socket,
    loginlvl => "anonymous",
    selected => 0,
  };
  return 0 if($self->_getSingleLine() ne PROMPT);
  return $self;
}

# Object method: execute()
# Executes a given command on the server and returns the output in an array
#
# EXAMPLE:
#   my @output = $obj->execute("pl");
#
sub execute {
  my ($self, $command) = @_;
  $self->_sendCommand($command);
  return $self->_getMultipleLines();
}

# Object method: selectServer()
# Selects a virtual server and returns TRUE or FALSE accordingly
#
# EXAMPLE:
#   $obj->selectServer("8767");
#
sub selectServer {
  my ($self, $port) = @_;
  $self->_sendCommand("sel $port");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  $self->{selected} = $port;
  return 1;
}

# Object method: loginSuperadmin()
# Authenticates as SuperAdmin and returns TRUE or FALSE accordingly
#
# EXAMPLE:
#   $obj->loginSuperadmin("superadmin","password");
#
sub loginSuperadmin {
  my ($self, $username, $password) = @_;
  $self->_sendCommand("slogin $username $password");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  $self->{loginlvl} = "superadmin";
  return 1;
}

# Object method: loginServeradmin()
# Authenticates as ServerAdmin and returns TRUE or FALSE accordingly
#
# EXAMPLE:
#   $obj->loginServeradmin("admin","password");
#
sub loginServeradmin {
  my ($self, $username, $password) = @_;
  $self->_sendCommand("login $username $password");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  $self->{loginlvl} = "serveradmin";
  return 1;
}

# Object method: getServerlist()
# Returns a list of UDP ports used by virtual TeamSpeak 2 servers where each
# element contains a TeamSpeak2::TCPquery::Server object
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       server_udpport => '8767',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @serverlist = $obj->getServerlist();
#   foreach(@serverlist) {
#     print "Found virtual server on port $_->{server_udpport}\n";
#   }
#
sub getServerlist {
  my $self = shift;
  $self->_sendCommand("sl");
  @serverlist = ();
  foreach($self->_getMultipleLines()) {
    if(/^(\d+)\s*$/) {
      push(@serverlist, TeamSpeak2::TCPquery::Server->new($self, $1, 1));
    }
  }
  return @serverlist;
}

# Object method: getDbServerlist()
# Returns a list of virtual TeamSpeak servers with their name and status
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       id      => '1',
#       udpport => '8767',
#       name    => 'TeamSpeak Public -1-',
#       status  => '1',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @serverlist = $obj->getDbServerlist();
#   foreach(@serverlist) {
#     print "Found virtual server on port $_->{udpport}\n";
#   }
#
sub getDbServerlist {
  my $self = shift;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("dbserverlist");
  @dbserverlist = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(/^(\d+)\t(\d+)\t"([^\r\n]*)"\t([01]+)\s*$/) {
        push(@dbserverlist, {
          id      => $1,
          udpport => $2,
          name    => $3,
          status  => $4,
        });
      }
    }
    $valid = 1 if($_ eq TITLE->{dbserverlist});
  }
  return @dbserverlist;
}

# Object method: getPlayerlist()
# Returns a list of players connected to the selected virtual server, where
# each element contains a TeamSpeak2::TCPquery::Player object
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       p_id      => '7432',
#       c_id      => '51427',
#       ps        => '19583',
#       bs        => '1725514',
#       pr        => '1109',
#       br        => '57671',
#       pl        => '3',
#       ping      => '32',
#       logintime => '10972',
#       idletime  => '172',
#       cprivs    => '1',
#       pprivs    => '0',
#       pflags    => '0',
#       ip        => '0.0.0.0',
#       nick      => 'Guest33',
#       loginname => '',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @playerlist = $obj->getPlayerlist();
#   foreach(@playerlist) {
#     print "Found player $_->{nick} with ID $_->{p_id}\n";
#   }
#
sub getPlayerlist {
  my $self = shift;
  $self->_sendCommand("pl");
  @playerlist = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(my @player = $_ =~ /^(\d+)\t([-1\d]+)\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t"(\d+\.\d+\.\d+\.\d+)"\t"([^\r\n]+)"\t"([^\r\n]*)"\s*$/) {
        push(@playerlist, TeamSpeak2::TCPquery::Player->new($self, @player));
      }
    }
    $valid = 1 if($_ eq TITLE->{pl});
  }
  return @playerlist;
}

# Object method: getChannellist()
# Returns a list of channels on the selected virtual server, where  each
# element contains a TeamSpeak2::TCPquery::Channel object
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       id       => '1',
#       codec    => '9',
#       parent   => '-1',
#       order    => '3200',
#       maxusers => '16',
#       name     => 'Default',
#       flags    => '16',
#       password => '0',
#       topic    => 'Default Channel',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @channellist = $obj->getChannellist();
#   foreach(@channellist) {
#     print "Found channel $_->{name} with ID $_->{id}\n";
#   }
#
sub getChannellist {
  my $self = shift;
  $self->_sendCommand("cl");
  @channellist = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(my @channel = $_ =~ /^(\d+)\t(\d+)\t([-1\d]+)\t(\d+)\t(\d+)\t"([^\r\n]+)"\t(\d+)\t([01]+)\t"([^\r\n]*)"\s*$/) {
        push(@channellist, TeamSpeak2::TCPquery::Channel->new($self, @channel));
      }
    }
    $valid = 1 if($_ eq TITLE->{cl});
  }
  return @channellist;
}

# Object method: getConnectionlist()
# Returns a list of open TCP query connections
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       login_name   => 'superadmin',
#       server_id    => '1',
#       ssa          => '1',
#       sa           => '1',
#       ip           => '0.0.0.0',
#       last_command => 'lc',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @connections = $obj->getConnectionlist();
#   foreach(@connections) {
#     print "User $_->{login_name} is connected from IP User $_->{ip}\n";
#   }
#
sub getConnectionlist {
  my $self = shift;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("lc");
  my @connections = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(/^"([^\r\n]+)"\t([01]+)\t([01]+)\t([01]+)\t"(\d+\.\d+\.\d+\.\d+)"\t"([^\r\n]+)"\s*$/) {
        push(@connections, {
          login_name   => $1,
          server_id    => $2,
          ssa          => $3,
          sa           => $4,
          ip           => $5,
          last_command => $6,
        });
      }
    }
    $valid = 1 if($_ eq TITLE->{lc});
  }
  return @connections;
}

# Object method: getBanlist()
# Returns a list of active bans from a virtual server
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       b_id    => '123',
#       ip      => '1.2.3.4',
#       created => '22-03-2007 22:23:42',
#       mins    => '0',
#       by      => 'IP Ban ScP[LoginName: ScP]',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @bans = $obj->getBanlist();
#   foreach(@bans) {
#     print "IP $_->{ip} got banned on $_->{created}\n";
#   }
#
sub getBanlist {
  my $self = shift;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("banlist");
  my @bans = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(/^(\d+)\t([^\r\n]+)\t($ini_date_format|never)\t(\d+)\t"([^\r\n]+)"\s*$/) {
        push(@bans, {
          b_id    => $1,
          ip      => $2,
          created => $3,
          mins    => $4,
          by      => $5,
        });
      }
    }
    $valid = 1 if($_ eq TITLE->{banlist});
  }
  return @bans;
}

# Object method: getGlobalinfo()
# Returns an array containing various information about the selected virtual
# server
#
# EXAMPLE ARRAY:
#   Array {
#     total_server_uptime   => '28D 02:15:16',
#     total_server_platform => 'Linux',
#     total_server_version  => '2.0.22.3',
#     total_servers         => '1',
#     total_users_online    => '228',
#     total_channels        => '103',
#     total_users_maximal   => '700',
#     total_bytessend       => '449361851104',
#     total_bytesreceived   => '209427557318',
#     total_packetssend     => '2037022851',
#     total_packetsreceived => '3496139082',
#     total_bwoutlastsec    => '153030',
#     total_bwinlastsec     => '87303',
#     total_bwoutlastmin    => '133123',
#     total_bwinlastmin     => '80786',
#     isp_ispname           => 'TeamSpeak',
#     isp_linkurl           => 'http://www.goteamspeak.com',
#     isp_adminemail        => 'noreply@goteamspeak.com',
#     isp_countrynumber     => '4428',
#   }
#
# EXAMPLE:
#   my $globalinfo = $obj->getGlobalinfo();
#   print "This server is hosted by $globalinfo->{isp_ispname}\n";
#
sub getGlobalinfo {
  my $self = shift;
  $self->_sendCommand("gi");
  my $info = ();
  foreach($self->_getMultipleLines()) {
    if(/^([\w\d_]+)=([^\r\n]*)\s*$/) {
      $info->{$1} = $2;
    }
  }
  return $info;
}

# Object method: getServerinfo()
# Returns an array containing various information about the selected virtual
# server
#
# EXAMPLE ARRAY:
#   Array {
#     server_id                        => '1',
#     server_udpport                   => '8767',
#     server_platform                  => 'Linux',
#     server_uptime                    => '2427711',
#     server_name                      => 'TeamSpeak Public -1-',
#     server_welcomemessage            => '[ Welcome to TeamSpeak, check www.GoTeamSpeak.com ]',
#     server_password                  => '0',
#     server_clan_server               => '0',
#     server_currentusers              => '240',
#     server_currentchannels           => '106',
#     server_maxusers                  => '700',
#     server_allow_codec_celp51        => '0',
#     server_allow_codec_celp63        => '0',
#     server_allow_codec_windowscelp52 => '0',
#     server_allow_codec_gsm148        => '0',
#     server_allow_codec_gsm164        => '0',
#     server_allow_codec_speex2150     => '0',
#     server_allow_codec_speex3950     => '0',
#     server_allow_codec_speex5950     => '0',
#     server_allow_codec_speex8000     => '-1',
#     server_allow_codec_speex11000    => '-1',
#     server_allow_codec_speex15000    => '-1',
#     server_allow_codec_speex18200    => '0',
#     server_allow_codec_speex24600    => '0',
#     server_bwoutlastsec              => '129595',
#     server_bwinlastsec               => '71923',
#     server_bwoutlastmin              => '114223',
#     server_bwinlastmin               => '73959',
#     server_packetssend               => '3496571240',
#     server_packetsreceived           => '2037312007',
#     server_bytessend                 => '449413763507',
#     server_bytesreceived             => '209457709911',
#     server_webpost_linkurl           => '',
#     server_webpost_posturl           => '',
#   }
#
# EXAMPLE:
#   my $serverinfo = $obj->getServerinfo();
#   print "The servers name is $serverinfo->{server_name}\n";
#
sub getServerinfo {
  my $self = shift;
  $self->_sendCommand("si");
  my $info = ();
  foreach($self->_getMultipleLines()) {
    if(/^([\w\d_]+)=([^\r\n]*)\s*$/) {
      $info->{$1} = $2;
    }
  }
  return $info;
}

# Object method: getPlayerinfo()
# Returns an array containing various information about the player with the
# given ID
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       p_id      => '63098',
#       p_dbid    => '0',
#       c_id      => '38340',
#       nickname  => 'Jun-Kit',
#       loginname => '',
#       ip        => '83.146.55.221',
#       sa        => '0',
#       st        => '0',
#     },
#     [1] => {
#       c_id => '38340',
#       name => 'MRLR',
#       ca   => '1',
#       ao   => '0',
#       av   => '0',
#       o    => '0',
#       v    => '0',
#     },
#     [2] => ...
#   }
#
# EXAMPLE:
#   my @playerinfo = $obj->getPlayerinfo(63098);
#   print "Player $playerinfo[0]->{nickname} has ID $playerinfo[0]->{p_id}\n";
#
sub getPlayerinfo {
  my ($self, $player_id) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("pi $player_id");
  my @info = ();
  my $valid_1 = 0;
  my $valid_2 = 0;
  foreach($self->_getMultipleLines()) {
    $valid_2 = 1 if($_ eq TITLE->{pi_2});
    if($valid_2) {
      if(/^(\d+)\t"([^\r\n]+)"\t([01]+)\t([01]+)\t([01]+)\t([01]+)\t([01]+)\s*$/) {
        push(@info, {
          c_id => $1,
          name => $2,
          ca   => $3,
          o    => $4,
          ao   => $5,
          v    => $6,
          av   => $7,
        });
      }
    } elsif($valid_1) {
      if(/^(\d+)\t(\d+)\t(\d+)\t"([^\r\n]+)"\t"([^\r\n]*)"\t([01]+)\t([01]+)\t"(\d+\.\d+\.\d+\.\d+)"\s*$/) {
        push(@info, {
          p_id      => $1,
          p_dbid    => $2,
          c_id      => $3,
          nickname  => $4,
          loginname => $5,
          sa        => $6,
          st        => $7,
          ip        => $8,
        });
      }
    }
    $valid_1 = 1 if($_ eq TITLE->{pi_1});
  }
  return @info;
}

# Object method: getChannelinfo()
# Returns an array containing various information about the channel with the
# given ID
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       c_id          => '38340',
#       c_pid         => '-1',
#       c_dbid        => '0',
#       c_name        => '[ WC3 Pro Gaming ]',
#       c_fU          => '1',
#       c_fM          => '0',
#       c_fP          => '1',
#       c_fH          => '0',
#       c_fD          => '0',
#       c_codec       => '9',
#       c_order       => '3200',
#       c_maxusers    => '700',
#       c_created     => '30-07-2006 14:45:01',
#       c_topic       => '',
#       c_description => '',
#     },
#     [1] => {
#       p_id   => '63098',
#       p_nick => 'Jun-Kit',
#       sa     => '0',
#       ca     => '1',
#       o      => '0',
#       ao     => '0',
#       v      => '0',
#       av     => '0',
#       cst    => '0',
#       reg    => '1',
#     },
#     [2] => ...
#   }
#
# EXAMPLE:
#   my @channelinfo = $obj->getChannelinfo(38340);
#   print "Channel $channelinfo[0]->{c_name} has DBID $channelinfo[0]->{c_dbid}\n";
#
sub getChannelinfo {
  my ($self, $channel_id) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("ci $channel_id");
  my @info = ();
  my $valid_1 = 0;
  my $valid_2 = 0;
  my $multiline_desc = 0;
  foreach($self->_getMultipleLines()) {
    $valid_2 = 1 if($_ eq TITLE->{ci_2});
    if($valid_2) {
      if(/^(\d+)\t"([^\r\n]+)"\t([01]+)\t([01]+)\t([01]+)\t([01]+)\t([01]+)\t([01]+)\t([01]+)\t([01]+)\s*$/) {
        push(@info, {
          p_id   => $1,
          p_nick => $2,
          sa     => $3,
          ca     => $4,
          o      => $5,
          ao     => $6,
          v      => $7,
          av     => $8,
          cst    => $9,
          reg    => $10,
        });
      }
    } elsif($valid_1) {
      if(/^(\d+)\t([-1\d]+)\t(\d+)\t"([^\r\n]+)"\t([01]+)\t([01]+)\t([01]+)\t([01]+)\t([01]+)\t(\d+)\t(\d+)\t(\d+)\t(\d\d-\d\d-\d\d\s\d\d:\d\d:\d\d)\t"([^\r\n]*)"\t"([^\r\n]*)"\s*$/) {
        push(@info, {
          c_id          => $1,
          c_pid         => $2,
          c_dbid        => $3,
          c_name        => $4,
          c_fU          => $5,
          c_fM          => $6,
          c_fP          => $7,
          c_fH          => $8,
          c_fD          => $9,
          c_codec       => $10,
          c_order       => $11,
          c_maxusers    => $12,
          c_created     => $13,
          c_topic       => $14,
          c_description => $15,
        });
      } elsif(/^(\d+)\t([-1\d]+)\t(\d+)\t"([^\r\n]+)"\t([01]+)\t([01]+)\t([01]+)\t([01]+)\t([01]+)\t(\d+)\t(\d+)\t(\d+)\t(\d\d-\d\d-\d\d\s\d\d:\d\d:\d\d)\t"([^\r\n]*)"\t"([^\r\n]*)\s*$/) {
        $multiline_desc = 1;
        push(@info, {
          c_id          => $1,
          c_pid         => $2,
          c_dbid        => $3,
          c_name        => $4,
          c_fU          => $5,
          c_fM          => $6,
          c_fP          => $7,
          c_fH          => $8,
          c_fD          => $9,
          c_codec       => $10,
          c_order       => $11,
          c_maxusers    => $12,
          c_created     => $13,
          c_topic       => $14,
          c_description => $15,
        });
      } else {
        $info[0]->{c_description} .= "\n$_" if($multiline_desc);
      }
    }
    $valid_1 = 1 if($_ eq TITLE->{ci_1});
  }  
  if($multiline_desc) {
    $/ = "\"\t\n";
    chomp($info[0]->{c_description});
  }
  return @info;
}

# Object method: getRegistrations()
# Returns a list of registrations on a virtual TeamSpeak 2 server
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       id         => '43',
#       loginname  => 'SA_Account3'
#       sa         => '0',
#       created    => '22-12-2005',
#       lastonline => 'never',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @registrations = $obj->getRegistrations();
#   foreach my $account (@registrations) {
#     print "Account $account->{p_dbid} was created on $account->{created}\n";
#   }
#
sub getRegistrations {
  my ($self, $username) = @_;
  $self->_checkLoginLvl("serveradmin",(caller(0))[3]);
  $self->_sendCommand("dbuserlist");
  my @results = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(/^(\d+)\t(-1|0)\t($ini_date_format)\t($ini_date_format|never)\t"([^\r\n]+)"\s*$/) {
        push(@results, {
          id         => $1,
          sa         => $2,
          created    => $3,
          lastonline => $4,
          loginname  => $5,
        });
      }
    }
    $valid = 1 if($_ eq TITLE->{dbuserlist});
  }
  return @results;
}

# Object method: getSuperadminRegistrations()
# Returns a list of superadmin accounts on a TeamSpeak 2 server
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       id         => '43',
#       loginname  => 'SA_Account3'
#       sa         => '0',
#       created    => '22-12-2005',
#       lastonline => 'never',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @registrations = $obj->getSuperadminRegistrations();
#   foreach my $account (@registrations) {
#     print "Account $account->{p_dbid} was created on $account->{created}\n";
#   }
#
sub getSuperadminRegistrations {
  my ($self, $username) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("dbsuserlist");
  my @results = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(/^(\d+)\t($ini_date_format)\t($ini_date_format|never)\t"([^\r\n]+)"\s*$/) {
        push(@results, {
          id         => $1,
          created    => $2,
          lastonline => $3,
          loginname  => $4,
        });
      }
    }
    $valid = 1 if($_ eq TITLE->{dbsuserlist});
  }
  return @results;
}

# Object method: getChannelsWithPlayers()
# Returns a list of Channels with players
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       c_id   => '1',
#       c_name => 'Default Channel',
#       p_id   => '1',
#       c_name => 'Guest1',
#     },
#     [1] => {
#       c_id   => '1',
#       c_name => 'Default Channel',
#       p_id   => '2',
#       c_name => 'Guest2',
#     },
#     [2] => ...
#   }
#
# EXAMPLE:
#   my @dccl = $obj->getChannelsWithPlayers();
#   foreach my $thing (@dccl) {
#     print "Player $thing->{p_name} is in channel $thing->{c_name}\n";
#   }
#
sub getChannelsWithPlayers {
  my ($self, $username) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("dccl");
  my @results = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(/^(\d+)\t"([^\r\n]+)"\t(\d+)\t"([^\r\n]+)"\s*$/) {
        push(@results, {
          c_id   => $1,
          c_name => $2,
          p_id   => $3,
          p_name => $4,
        });
      }
    }
    $valid = 1 if($_ eq TITLE->{dccl});
  }
  return @results;
}

# Object method: getServerversion()
# Returns an array containing various information about the selected virtual
# server
#
# EXAMPLE ARRAY:
#   Array {
#     version  => '2.0.22.3',
#     major    => '2'
#     minor    => '0',
#     release  => '22',
#     build    => '3',
#     platform => 'Linux',
#     license  => 'Freeware',
#   }
#
# EXAMPLE:
#   my $serverversion = $obj->getServerversion();
#   print "This server is running on $serverversion->{platform}\n";
#
sub getServerversion {
  my $self = shift;
  $self->_sendCommand("ver");
  my $version = ();
  foreach($self->_getMultipleLines()) {
    if(/^((\d+)\.(\d+)\.(\d+)\.(\d+))\s(\w+)\s(\w+)\s*$/) {
      $version->{version}  = $1;
      $version->{major}    = $2;
      $version->{minor}    = $3;
      $version->{release}  = $4;
      $version->{build}    = $5;
      $version->{platform} = $6;
      $version->{license}  = $7;
    }
  }
  return $version;
}

# Object method: getServerlog()
# Returns an array containing various information from the servers logfile
#
# EXAMPLE ARRAY:
#   Array {
#     date  => '22-06-2006',
#     time  => '17:41:38'
#     type  => 'server',
#     s_id  => '0',
#     event => 'Started VirtualServer id:16 with port:9999',
#   }
#
# EXAMPLE:
#   my @serverlog = $obj->getServerlog();
#   foreach my $line (@serverlog) {
#     print "Event $line->{event} logged on server ID $line->{s_id}\n";
#   }
#
sub getServerlog {
  my ($self, $lines) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  if(defined($lines)) {
    $self->_sendCommand("log $lines");
  } else {
    $self->_sendCommand("log");
  }
  my @log = ();
  foreach($self->_getMultipleLines()) {
    if(/^(\d+-\d+-\d+)\s+(\d+:\d+:\d+),\w+,\w+,([\s\w]+),\s+SID:\s+(\d+)\s+(.+)\s*$/) {
      push(@log, {
        date  => $1,
        time  => $2,
        type  => $3,
        s_id  => $4,
        event => $5,
      });
    } elsif(/^(\d+-\d+-\d+)\s+(\d+:\d+:\d+),\w+,\w+,([\s\w]+),\s+([^\r\n]+)\s*$/) {
      push(@log, {
        date  => $1,
        time  => $2,
        type  => $3,
        s_id  => 0,
        event => $4,
      });
    } elsif(/^-{13,14}\s+log\s(\w+)\sat\s(\d+-\d+-\d+)\s(\d+:\d+)\s+-{13,14}\s*$/) {
      push(@log, {
        date  => $2,
        time  => $3.":00",
        type  => "log",
        s_id  => 0,
        event => "Log ".$1,
      });
    }
  }
  return @log;
}

# Object method: getPacketloss()
# Returns a variable containing the servers packetloss as a string
#
# EXAMPLE VARIABLE:
#   Var = '0,31%'
#
# EXAMPLE:
#   my $packetloss = $obj->getPacketloss();
#   print "This servers packetloss is $packetloss\n";
#
sub getPacketloss {
  my $self = shift;
  $self->_sendCommand("gapl");
  foreach($self->_getMultipleLines()) {
    if(/^average_packet_loss=(\d+,\d+%)\s*$/) {
      return $1;
    }
  }
  return 0;
}

# Object method: markLog()
# Adds a new line into the servers logfile and returns TRUE or FALSE accordingly
#
# EXAMPLE:
#   $obj->markLog("test");
#
sub markLog {
  my ($self, $message) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("logmark $message");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: setHosterbanner()
# Sets the servers hosterbanner and returns TRUE or FALSE accordingly
#
# EXAMPLE:
#   $obj->setHosterbanner("http://domain.tld/banner.jpg");
#
sub setHosterbanner {
  my ($self, $value) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("globalset hoster_gfx_url $value");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: setAllowedNicknameChars()
# Sets the allowed chars for nicknames returns TRUE or FALSE accordingly
#
# EXAMPLE:
#   $obj->setAllowedNicknameChars("ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890");
#
sub setAllowedNicknameChars {
  my ($self, $value) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("globalset allowedclientnamechars $value");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: setDisallowedNicknameChars()
# Sets the disallowed chars for nicknames returns TRUE or FALSE accordingly
#
# EXAMPLE:
#   $obj->setDisallowedNicknameChars("[](){}");
#
sub setDisallowedNicknameChars {
  my ($self, $value) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("globalset disallowedclientnamechars $value");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: emptyBanlist()
# Deletes all bans from a virtual servers banlist
#
# EXAMPLE:
#   $obj->emptyBanlist();
#
sub emptyBanlist {
  my $self = shift;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("banclear");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: addBan()
# Adds a ban to a virtual servers banlist
#
# EXAMPLE:
#   $obj->addBan("1.2.3.4");
#
sub addBan {
  my ($self, $id, $minutes) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  if(defined($minutes)) {
    $self->_sendCommand("banadd $id $minutes");
  } else {
    $self->_sendCommand("banadd $id");
  }
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: deleteBan()
# Deletes a ban from a virtual servers banlist
#
# EXAMPLE:
#   $obj->deleteBan("1");
#
sub deleteBan {
  my ($self, $id) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("bandel $id");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: addSuperadminRegistration()
# Adds a new superadmin account to the servers database
#
# EXAMPLE:
#   $obj->addSuperadminRegistration("username","password");
#
sub addSuperadminRegistration {
  my ($self, $username, $password) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("dbsuseradd $username $password $password");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: addRegistration()
# Adds a new account to the servers database
#
# EXAMPLE:
#   $obj->addRegistration("username","password","1");
#
sub addRegistration {
  my ($self, $username, $password, $serveradmin) = @_;
  $self->_checkLoginLvl("serveradmin",(caller(0))[3]);
  $self->_sendCommand("dbuseradd $username $password $password $serveradmin");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: addServer()
# Adds a new virtual TeamSpeak server to the database
#
# EXAMPLE:
#   $obj->addServer(8768);
#
sub addServer {
  my ($self, $port) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("serveradd $port");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: deleteSuperadminRegistration()
# Removes a superadmin account from the servers database
#
# EXAMPLE:
#   $obj->deleteSuperadminRegistration(1);
#
sub deleteSuperadminRegistration {
  my ($self, $id) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("dbsuserdel $id");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: deleteRegistration()
# Removes an account from the servers database
#
# EXAMPLE:
#   $obj->deleteRegistration(1);
#
sub deleteRegistration {
  my ($self, $id) = @_;
  $self->_checkLoginLvl("serveradmin",(caller(0))[3]);
  $self->_sendCommand("dbuserdel $id");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: deleteServer()
# Deletes a new virtual TeamSpeak server from the database
#
# EXAMPLE:
#   $obj->deleteServer(2);
#
sub deleteServer {
  my ($self, $id) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("serverdel $id");
  return 0 if($self->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: findPlayer()
# Finds players on a virtual TeamSpeak 2 server based on a string
#
# EXAMPLE ARRAY:
#   Array {
#     p_id      => '323',
#     p_dbid    => '0'
#     c_id      => '21',
#     nickname  => 'Guest33',
#     loginname => '',
#     ip        => '0.0.0.0',
#   }
#
# EXAMPLE:
#   my @players = $obj->findPlayer("guest");
#   foreach my $player (@players) {
#     print "Found channel $player->{nickname}\n";
#   }
#
sub findPlayer {
  my ($self, $nickname) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("fp $nickname");
  my @results = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(/^(\d+)\t(\d+)\t([-1\d]+)\t"([^\r\n]+)"\t"([^\r\n]*)"\t"(\d+\.\d+\.\d+\.\d+)"\s*$/) {
        push(@results, {
          p_id      => $1,
          p_dbid    => $2,
          c_id      => $3,
          nickname  => $4,
          loginname => $5,
          ip        => $6,
        });
      }
    }
    $valid = 1 if($_ eq TITLE->{fp});
  }
  return @results;
}

# Object method: findChannel()
# Finds channels on a virtual TeamSpeak 2 server based on a string
#
# EXAMPLE ARRAY:
#   Array {
#     c_id   => '1',
#     c_dbid => '1'
#     c_name => 'Default Channel',
#   }
#
# EXAMPLE:
#   my @channels = $obj->findChannel("default");
#   foreach my $channel (@channels) {
#     print "Found channel $channel->{c_name}\n";
#   }
#
sub findChannel {
  my ($self, $name) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("fc $name");
  my @results = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(/^(\d+)\t(\d+)\t([^\r\n]+)\s*$/) {        
        push(@results, {
          c_id      => $1,
          c_dbid    => $2,
          c_name    => $3,
        });
      }
    }
    $valid = 1 if($_ eq TITLE->{fc});
  }
  return @results;
}

# Object method: findRegistration()
# Finds registrations on a virtual TeamSpeak 2 server based on a string
#
# EXAMPLE ARRAY:
#   Array {
#     p_dbid     => '43',
#     loginname  => 'SA_Account3'
#     sa         => '0',
#     created    => '22-12-2005',
#     lastonline => 'never',
#   }
#
# EXAMPLE:
#   my @registrations = $obj->findRegistration("SA");
#   foreach my $account (@registrations) {
#     print "Account $account->{p_dbid} was created on $account->{created}\n";
#   }
#
sub findRegistration {
  my ($self, $username) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("dbfp $username");
  my @results = ();
  my $valid = 0;
  foreach($self->_getMultipleLines()) {
    if($valid) {
      if(/^(\d+)\t"([^\r\n]+)"\t(-1|0)\t($ini_date_format)\t($ini_date_format|never)\s*$/) {
        push(@results, {
          p_dbid     => $1,
          loginname  => $2,
          sa         => $3,
          created    => $4,
          lastonline => $5,
        });
      }
    }
    $valid = 1 if($_ eq TITLE->{dbfp});
  }
  return @results;
}

# Object method: findServerlog()
# Finds entries in the servers logfile based on a string
#
# EXAMPLE ARRAY:
#   Array {
#     date  => '22-06-2006',
#     time  => '17:41:38'
#     type  => 'server',
#     s_id  => '0',
#     event => 'Started VirtualServer id:16 with port:9999',
#   }
#
# EXAMPLE:
#   my @serverlog = $obj->findServerlog("Started");
#   foreach my $line (@serverlog) {
#     print "Found event $line->{event} on server ID $line->{s_id}\n";
#   }
#
sub findServerlog {
  my ($self, $pattern) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_sendCommand("logfind $pattern");
  my @results = ();
  foreach($self->_getMultipleLines()) {
    if(/^(\d+-\d+-\d+)\s+(\d+:\d+:\d+),\w+,\w+,([\s\w]+),\s+SID:\s+(\d+)\s+(.+)\s*$/) {
      push(@results, {
        date  => $1,
        time  => $2,
        type  => $3,
        s_id  => $4,
        event => $5,
      });
    } elsif(/^(\d+-\d+-\d+)\s+(\d+:\d+:\d+),\w+,\w+,([\s\w]+),\s+([^\r\n]+)\s*$/) {
      push(@results, {
        date  => $1,
        time  => $2,
        type  => $3,
        s_id  => 0,
        event => $4,
      });
    } elsif(/^-{13,14}\s+log\s(\w+)\sat\s(\d+-\d+-\d+)\s(\d+:\d+)\s+-{13,14}\s*$/) {
      push(@results, {
        date  => $2,
        time  => $3.":00",
        type  => "log",
        s_id  => 0,
        event => "Log ".$1,
      });
    }
  }
  return @results;
}

# Object method: sendMessage()
# Sends one ore more messages to all players on all servers and returns TRUE
# or FALSE accordingly
#
sub sendMessage {
  my ($self, @messages) = @_;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  foreach(@messages) {
    $self->_sendCommand("msgall @".$_);
    return 0 if($self->_getSingleLine() ne SUCCESS);
  }
  return 1;
}

# Object method: getLastPlayerlist()
# Returns the last created list of players connected to the selected virtual
# server or creates a new one by calling getPlayerlist() if none exists yet
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       p_id      => '7432',
#       c_id      => '51427',
#       ps        => '19583',
#       bs        => '1725514',
#       pr        => '1109',
#       br        => '57671',
#       pl        => '3',
#       ping      => '32',
#       logintime => '10972',
#       idletime  => '172',
#       cprivs    => '1',
#       pprivs    => '0',
#       pflags    => '0',
#       ip        => '0.0.0.0',
#       nick      => 'Guest33',
#       loginname => '',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @playerlist = $obj->getLastPlayerlist();
#   foreach(@playerlist) {
#     print "Found player $_->{nick} with ID $_->{p_id}\n";
#   }
#
sub getLastPlayerlist {
  my $self = shift;
  if(!@playerlist) {
    @playerlist = $self->getPlayerlist();
  } else {
    if($self->{selected} != $playerlist[0]->{server}) {
      @playerlist = $self->getPlayerlist();
    }
  }
  return @playerlist;
}

# Object method: getLastChannellist()
# Returns the last created list of channels on the selected virtual server
# or creates a new one by calling getChannellist() if none exists yet
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       id       => '1',
#       codec    => '9',
#       parent   => '-1',
#       order    => '3200',
#       maxusers => '16',
#       name     => 'Default',
#       flags    => '16',
#       password => '0',
#       topic    => 'Default Channel',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @channellist = $obj->getLastChannellist();
#   foreach(@channellist) {
#     print "Found channel $_->{name} with ID $_->{id}\n";
#   }
#
sub getLastChannellist {
  my $self = shift;  
  if(!@channellist) {
    @channellist = $self->getChannellist();
  } else {
    if($self->{selected} != $channellist[0]->{server}) {
      @channellist = $self->getChannellist();
    }
  }
  return @channellist;
}

# Object method: getLastServerlist()
# Returns the last created list of virtual servers or creates a new one by
# calling getServerlist() if none exists yet
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => '8767',
#     [1] => '8768',
#     [2] => '8769',
#     [3] => ...
#   }
#
# EXAMPLE:
#   my @serverlist = $obj->getLastServerlist();
#   foreach(@serverlist) {
#     print "Found virtual server on port $_\n";
#   }
#
sub getLastServerlist {
  my $self = shift;
  @serverlist = $self->getServerlist() if(!@serverlist);
  return @serverlist;
}

# Object method: getLastDbServerlist()
# Returns the last created list of virtual servers or creates a new one by
# calling getDbServerlist() if none exists yet
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => {
#       id      => '1',
#       udpport => '8767',
#       name    => 'TeamSpeak Public -1-',
#       status  => '1',
#     },
#     [1] => ...
#   }
#
# EXAMPLE:
#   my @serverlist = $obj->getLastDbServerlist();
#   foreach(@serverlist) {
#     print "Found virtual server on port $_->{udpport}\n";
#   }
#
sub getLastDbServerlist {
  my $self = shift;
  $self->_checkLoginLvl("superadmin",(caller(0))[3]);
  @dbserverlist = $self->getDbServerlist() if(!@dbserverlist);
  return @dbserverlist;
}

# Object method: getPlayerByID()
# Returns a player object containing all information about the player
# with the given ID
#
# EXAMPLE ARRAY:
#   Array {
#     p_id      => '7432',
#     c_id      => '51427',
#     ps        => '19583',
#     bs        => '1725514',
#     pr        => '1109',
#     br        => '57671',
#     pl        => '3',
#     ping      => '32',
#     logintime => '10972',
#     idletime  => '172',
#     cprivs    => '1',
#     pprivs    => '0',
#     pflags    => '0',
#     ip        => '0.0.0.0',
#     nick      => 'Guest33',
#     loginname => '',
#   }
#
# EXAMPLE:
#   $player = $ts2->getPlayerByID("7432");
#
sub getPlayerByID {
  my ($self, $player_id) = @_;
  foreach my $player ($self->getLastPlayerlist()) {
    return $player if($player->{p_id} == $player_id);
  }
  return 0;
}

# Object method: getPlayerByName()
# Returns a player object containing all information about the player with
# the given nickname
#
# EXAMPLE ARRAY:
#   Array {
#     p_id      => '7432',
#     c_id      => '51427',
#     ps        => '19583',
#     bs        => '1725514',
#     pr        => '1109',
#     br        => '57671',
#     pl        => '3',
#     ping      => '32',
#     logintime => '10972',
#     idletime  => '172',
#     cprivs    => '1',
#     pprivs    => '0',
#     pflags    => '0',
#     ip        => '0.0.0.0',
#     nick      => 'Guest33',
#     loginname => '',
#   }
#
# EXAMPLE:
#   $player = $ts2->getPlayerByName("Guest33");
#
sub getPlayerByName {
  my ($self, $player_name) = @_;
  foreach my $player ($self->getLastPlayerlist()) {
    return $player if($player->{nick} eq $player_name);
  }
  return 0;
}

# Object method: getChannelByID()
# Returns a channel object containing all information about the channel with
# the given ID
#
# EXAMPLE ARRAY:
#   Array {
#     id       => '1',
#     codec    => '9',
#     parent   => '-1',
#     order    => '3200',
#     maxusers => '16',
#     name     => 'Default',
#     flags    => '16',
#     password => '0',
#     topic    => 'Default Channel',
#   }
#
# EXAMPLE:
#   $channel = $ts2->getChannelByID("1");
#
sub getChannelByID {
  my ($self, $channel_id) = @_;
  foreach my $channel ($self->getLastChannellist()) {
    return $channel if($channel->{id} == $channel_id);
  }
  return 0;
}

# Object method: getChannelByName()
# Returns a channel object containing all information about the channel with
# the given name
#
# EXAMPLE ARRAY:
#   Array {
#     id       => '1',
#     codec    => '9',
#     parent   => '-1',
#     order    => '3200',
#     maxusers => '16',
#     name     => 'Default',
#     flags    => '16',
#     password => '0',
#     topic    => 'Default Channel',
#   }
#
# EXAMPLE:
#   $channel = $ts2->getChannelByID("1");
#
sub getChannelByName {
  my ($self, $channel_name) = @_;
  foreach my $channel ($self->getLastChannellist()) {
    return $channel if($channel->{name} eq $channel_name);
  }
  return 0;
}

# Object method: getServer()
# Returns a server object containing all information about the server with
# the given UDP port
#
# EXAMPLE ARRAY:
#   Array {
#     server_id                        => '1',
#     server_udpport                   => '8767',
#     server_platform                  => 'Linux',
#     server_uptime                    => '2427711',
#     server_name                      => 'TeamSpeak Public -1-',
#     server_welcomemessage            => '[ Welcome to TeamSpeak, check www.GoTeamSpeak.com ]',
#     server_password                  => '0',
#     server_clan_server               => '0',
#     server_currentusers              => '240',
#     server_currentchannels           => '106',
#     server_maxusers                  => '700',
#     server_allow_codec_celp51        => '0',
#     server_allow_codec_celp63        => '0',
#     server_allow_codec_windowscelp52 => '0',
#     server_allow_codec_gsm148        => '0',
#     server_allow_codec_gsm164        => '0',
#     server_allow_codec_speex2150     => '0',
#     server_allow_codec_speex3950     => '0',
#     server_allow_codec_speex5950     => '0',
#     server_allow_codec_speex8000     => '-1',
#     server_allow_codec_speex11000    => '-1',
#     server_allow_codec_speex15000    => '-1',
#     server_allow_codec_speex18200    => '0',
#     server_allow_codec_speex24600    => '0',
#     server_bwoutlastsec              => '129595',
#     server_bwinlastsec               => '71923',
#     server_bwoutlastmin              => '114223',
#     server_bwinlastmin               => '73959',
#     server_packetssend               => '3496571240',
#     server_packetsreceived           => '2037312007',
#     server_bytessend                 => '449413763507',
#     server_bytesreceived             => '209457709911',
#     server_webpost_linkurl           => '',
#     server_webpost_posturl           => '',
#   }
#
# EXAMPLE:
#   my $server = $obj->getServer();
#   print "The servers ID is $server->{server_id}\n";
#
sub getServer {
  my ($self, $server_port) = @_;
  my $server = 0;
  foreach($self->getLastServerlist()) {
    $server = $_ if($_->{server_udpport} eq $server_port);
  }
  if(!$server) {
    foreach($self->getLastDbServerlist()) {
      if($_->{udpport} == $server_port) {
        $server = TeamSpeak2::TCPquery::Server->new($self, $_->{udpport}, 0);
        $server->{server_id} = $_->{id};
        $server->{server_name} = $_->{name};
      } 
    }
  }
  if($server->{status}) {
    if($server && $self->{selected} != $server_port) {
      $self->selectServer($server_port) || return 0;
    }
    my $serverinfo = $self->getServerinfo();
    if($serverinfo) {
      foreach(keys(%$serverinfo)) {
        $server->{$_} = $serverinfo->{$_};
      }
    }
  }
  return $server;
}

# Object method: debug()
# Returns an array containing various debug information
#
# EXAMPLE ARRAY:
#   Array {
#     [0] => '<< [TS]',
#     [1] => '>> ver',
#     [2] => '<< 2.0.20.1 Linux Freeware',
#     [3] => '<< OK',
#     [4] => ...
#   }
#
# EXAMPLE:
#   my @debug = $obj->debug();
#   foreach(@debug) {
#     print "DEBUG: $_\n";
#   }
#
sub debug {
  my $self = shift;
  return @debug;
}

# Object method: toggleDebug()
# Enables or disables debugging while the script is running
#
# EXAMPLE:
#   $obj->toggleDebug();
#
sub toggleDebug {
  my $self = shift;
  if(PACKAGE->{debugging}) {
    PACKAGE->{debugging} = 0;
  } else {
    PACKAGE->{debugging} = 1;
  }
  return 1;
}

# Object method: setDateTimeFormat()
# Allows you to change the date and time search pattern if the "DateTimeFormat"
# setting has been changed in the servers configuration file
#
# EXAMPLE:
#   $obj->setDateTimeFormat("dd-mm-yyyy hh:nn:ss");
#
sub setDateTimeFormat {
  my $self = shift;
  my $pattern = shift;
  $pattern =~ s/\w/\\d/g;
  $pattern =~ s/\s/\\s/g;
  print $pattern."\n";
  $ini_date_format = $pattern;
  return 1;
}

# Object method: sortPlayerlist()
# Returns a Client-like sorted playerlist array
#
# EXAMPLE:
#   my @sorted = $obj->sortPlayerlist(@unsorted);
#
sub sortPlayerlist {
  my ($self, @unsorted) = @_;
  my @sorted = sort {
    my $rank_a = $a->getFlagRanking();
    my $rank_b = $b->getFlagRanking();
    if($rank_a != $rank_b) {
      $rank_a <=> $rank_b;
    } else {
      uc($a->{nick}) cmp uc($b->{nick});
    }
  } (@unsorted);  
  return @sorted;
}

# Object method: sortChannellist()
# Returns a Client-like sorted channellist array
#
# EXAMPLE:
#   my @sorted = $obj->sortChannellist(@unsorted);
#
sub sortChannellist {
  my ($self, @unsorted) = @_;
  my @sorted = sort {
    if($a->{order} != $b->{order}) {
      $a->{order} <=> $b->{order};
    } else {
      uc($a->{name}) cmp uc($b->{name});
    }
  } (@unsorted);  
  return @sorted;
}

# ===========================================================================
# Various internal functions that should be never called directly
# ===========================================================================

# Object method: _getMultipleLines()
# Returns an array containing the servers output, where the each line is a
# seperate element of the array
#
sub _getMultipleLines {
  my $self = shift;
  my @lines = ();
  while(1) {
    my $line = $self->_getSingleLine();
    if($line eq SUCCESS || lc(substr($line,0,5)) eq ERROR->{unknown}) {
      last;
    }
    push(@lines, $line);
  }
  return @lines;
}

# Object method: _getSingleLine()
# Receives a single line from the TeamSpeak 2 server and removes any trailing
# newline characters
#
sub _getSingleLine {
  my $self = shift;
  $/ = "\r\n";
  my $line = readline($self->{handle}) || die("Connection to server (\"".$self->{handle}->peerhost()."\") lost");
  chomp($line);
  push(@debug, "<< $line") if(PACKAGE->{debugging});
  return $line;
}

# Object method: _sendCommand()
# Sends a single command to the TeamSpeak 2 server
#
sub _sendCommand {
  my ($self, $line) = @_;
  $self->{handle}->send("$line\n");
  $line = "slogin ********* *********" if(substr($line,0,6) eq "slogin");
  $line = "login ********* *********" if(substr($line,0,5) eq "login");
  push(@debug, ">> $line") if(PACKAGE->{debugging});
  return 1;
}

# Object method: _checkLoginLvl()
# Checks the users current login lvl and generates an exeption
#
sub _checkLoginLvl {
  my ($self, $required, $function) = @_;
  if($required eq "superadmin") {
    if($self->{loginlvl} ne $required) {
      die("Can't use function (\"$function\") because of insufficient permissions");
    }
  } elsif($required eq "serveradmin") {
    if($self->{loginlvl} ne $required && $self->{loginlvl} ne "superadmin") {
      die("Can't use function (\"$function\") because of insufficient permissions");
    }
  }
  return 1;
}

1;