# Server.pm

# ===========================================================================
# file         : Server.pm
# copyright    : 2005, 2006 by Sven 'ScP' Paulsen & Felix 'WeeZer' Siefert
# email        : scp@planetteamspeak.com & weezer@planetteamspeak.com
# version      : 0.2.6
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 675 Mass Ave, Cambridge, MA 02139, USA.
# ===========================================================================

package TeamSpeak2::TCPquery::Server;

use strict;
use warnings;
use diagnostics;

use Data::Dumper;

use constant PACKAGE => {
  "name"    => "TeamSpeak2::TCPquery::Server",
  "version" => "0.2.6",
};

use constant SUCCESS => "OK";

# ===========================================================================
# There should be no need to change anything below
# ===========================================================================

# Class method: new()
# Creates a new TeamSpeak2::TCPquery::Server
#
sub new {
  my ($class, $tcpquery, $port, $status) = @_;
  my $self = bless {
    tcpquery                         => $tcpquery,
    status                           => $status,
    server_id                        => undef,
    server_udpport                   => $port,
    server_platform                  => undef,
    server_uptime                    => undef,
    server_name                      => undef,
    server_welcomemessage            => undef,
    server_password                  => undef,
    server_clan_server               => undef,
    server_currentusers              => undef,
    server_currentchannels           => undef,
    server_maxusers                  => undef,
    server_allow_codec_celp51        => undef,
    server_allow_codec_celp63        => undef,
    server_allow_codec_windowscelp52 => undef,
    server_allow_codec_gsm148        => undef,
    server_allow_codec_gsm164        => undef,
    server_allow_codec_speex2150     => undef,
    server_allow_codec_speex3950     => undef,
    server_allow_codec_speex5950     => undef,
    server_allow_codec_speex8000     => undef,
    server_allow_codec_speex11000    => undef,
    server_allow_codec_speex15000    => undef,
    server_allow_codec_speex18200    => undef,
    server_allow_codec_speex24600    => undef,
    server_bwoutlastsec              => undef,
    server_bwinlastsec               => undef,
    server_bwoutlastmin              => undef,
    server_bwinlastmin               => undef,
    server_packetssend               => undef,
    server_packetsreceived           => undef,
    server_bytessend                 => undef,
    server_bytesreceived             => undef,
    server_webpost_linkurl           => undef,
    server_webpost_posturl           => undef,
  };
  return $self;
}

# Object method: select()
# Selects the virtual server and returns TRUE or FALSE accordingly
#
sub select {
  my $self = shift;
  return 0 if(!$self->{tcpquery}->selectServer($self->{server_udpport}));
  return 1;
}

# Object method: sendMessage()
# Sends one ore more messages to all players on the server and returns TRUE
# or FALSE accordingly
#
sub sendMessage {
  my ($self, @messages) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  foreach(@messages) {
    $self->{tcpquery}->_sendCommand("msg @".$_);
    return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  }
  return 1;
}

# Object method: move()
# Moves all players on the server into a given channel and returns TRUE or
# FALSE accordingly
#
sub move {
  my ($self, $channel_id) = @_;
  $self->{tcpquery}->_checkLoginLvl("serveradmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->{tcpquery}->getLastPlayerlist()) {
    $errors = 1 if(!$player->move($channel_id));
  }
  return 0 if($errors);
  return 1;
}

# Object method: remove()
# Removes all players from the server and returns TRUE or FALSE accordingly
#
sub remove {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("serveradmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->{tcpquery}->getLastPlayerlist()) {
    $errors = 1 if(!$player->remove());
  }
  return 0 if($errors);
  return 1;
}

# Object method: kick()
# Kicks all players from the server and returns TRUE or FALSE accordingly
#
sub kick {
  my ($self, $reason) = @_;
  $self->{tcpquery}->_checkLoginLvl("serveradmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->{tcpquery}->getLastPlayerlist()) {
    $errors = 1 if(!$player->kick($reason));
  }
  return 0 if($errors);
  return 1;
}

# Object method: kickIdlers()
# kicks all idle players on the server and returns the number of kicked players
#
sub kickIdlers {
  my ($self, $idletime, $targets, $reason) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  $idletime = 15 if(!defined($idletime));
  $targets = 15 if(!defined($targets));
  if(defined($reason)) {
    $self->{tcpquery}->_sendCommand("ki $idletime $targets $reason");
  } else {
    $self->{tcpquery}->_sendCommand("ki $idletime $targets");
  }
  my $kicked = 0;
  foreach($self->{tcpquery}->_getMultipleLines()) {
    if(/^kicked=(\d+)\s*$/) {
      $kicked = $1;
    }
  }
  return $kicked;
}

# Object method: setServername()
# Sets the servers name and returns TRUE or FALSE accordingly
#
sub setServername {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  $self->{tcpquery}->_sendCommand("serverset server_name $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{server_name} = $value;
  return 1;
}

# Object method: setPassword()
# Sets the servers password and returns TRUE or FALSE accordingly
#
sub setPassword {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  $value = "#CLEAR#" if(!defined($value));
  $self->{tcpquery}->_sendCommand("serverset server_password $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{server_password} = 1 if($value ne "#CLEAR#");
  return 1;
}

# Object method: setWelcomemessage()
# Sets the servers welcome message and returns TRUE or FALSE accordingly
#
sub setWelcomemessage {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  $self->{tcpquery}->_sendCommand("serverset server_welcomemessage $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{server_welcomemessage} = $value;
  return 1;
}

# Object method: setMaxUsers()
# Sets the servers slot count and returns TRUE or FALSE accordingly
#
sub setMaxUsers {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  $self->{tcpquery}->_sendCommand("serverset server_maxusers $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{server_maxusers} = $value;
  return 1;
}

# Object method: setWebpostPostUrl()
# Sets the servers WebPost post URL and returns TRUE or FALSE accordingly
#
sub setWebpostPostUrl {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  $self->{tcpquery}->_sendCommand("serverset server_webpost_posturl $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{server_webpost_posturl} = $value;
  return 1;
}

# Object method: setWebpostLinkUrl()
# Sets the servers WebPost link URL and returns TRUE or FALSE accordingly
#
sub setWebpostLinkUrl {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  $self->{tcpquery}->_sendCommand("serverset server_webpost_linkurl $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{server_webpost_linkurl} = $value;
  return 1;
}

# Object method: setCodec()
# Defines whether the given codec is allowed and returns TRUE or FALSE
# accordingly
#
sub setCodec {
  my ($self, $codec, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  my $setting = "server_allow_codec_$codec";
  $self->{tcpquery}->_sendCommand("serverset $setting $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{$setting} = $value;
  return 1;
}

# Object method: setModePublic()
# Sets the servers into public mode and returns TRUE or FALSE accordingly
#
sub setModePublic {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  $self->{tcpquery}->_sendCommand("serverset server_clan_server 0");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{server_clan_server} = 0;
  return 1;
}

# Object method: setModeClan()
# Sets the servers into clan mode and returns TRUE or FALSE accordingly
#
sub setModeClan {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  $self->{tcpquery}->_sendCommand("serverset server_clan_server 1");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{server_clan_server} = 1;
  return 1;
}

# Object method: start()
# Starts the virtual server and returns TRUE or FALSE accordingly
#
sub start {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->{tcpquery}->_sendCommand("serverstart $self->{server_id}");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{status} = 1;
  return 1;
}

# Object method: stop()
# Stops the virtual server and returns TRUE or FALSE accordingly
#
sub stop {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->_throwException((caller(0))[3]);
  $self->{tcpquery}->_sendCommand("serverstop");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  $self->{status} = 0;
  return 1;
}

# Object method: delete()
# Deletes the virtual server and returns TRUE or FALSE accordingly
#
sub delete {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  my $server_id = 0;
  if(!defined($self->{server_id})) {
    foreach my $server ($self->{tcpquery}->getLastDbServerlist()) {
      $server_id = $server->{id} if($server->{udpport} == $self->{server_udpport});
    }
  } else {
    $server_id = $self->{server_id};
  }
  return 0 if(!$self->{tcpquery}->deleteServer($server_id));
  $self->{tcpquery}->{selected} = 0;
  return 1;
}

# Object method: getServerinfo()
# Returns an array containing various information about the server
#
sub getServerinfo {
  my $self = shift;
  $self->_throwException((caller(0))[3]);
  my $serverinfo = $self->{tcpquery}->getServerinfo();
  if($serverinfo) {
    foreach(keys(%$serverinfo)) {
      $self->{$_} = $serverinfo->{$_};
    }
  }
  return $serverinfo;
}

# Object method: getTopLevelChannels()
# Returns an array containing all top-level channels on the server
#
sub getTopLevelChannels {
  my $self = shift;
  $self->_throwException((caller(0))[3]);
  my @topchannels = ();
  foreach my $channel ($self->{tcpquery}->getLastChannellist) {
    if($channel->{parent} == -1) {
      push(@topchannels, $channel);
    }
  }
  return @topchannels;
}

# Object method: getSubLevelChannels()
# Returns an array containing all sub-level channels on the server
#
sub getSubLevelChannels {
  my $self = shift;
  $self->_throwException((caller(0))[3]);
  my @subchannels = ();
  foreach my $channel ($self->{tcpquery}->getLastChannellist) {
    if($channel->{parent} != -1) {
      push(@subchannels, $channel);
    }
  }
  return @subchannels;
}

# Object method: buildObjectTree()
# Returns a multi-dimensional array containing all channel and player objects
# of the virtual TeamSpeak server
#
sub buildObjectTree {
  my $self = shift;
  $self->getServerinfo() if(!defined($self->{server_platform}));
  $self->_throwException((caller(0))[3]);
  my $object = $self;
  my @channels = $self->{tcpquery}->sortChannellist($self->getTopLevelChannels());
  foreach my $channel (@channels) {
    my @subchannels = $self->{tcpquery}->sortChannellist($channel->getSubChannels());
    foreach my $subchannel (@subchannels) {
      my @subplayers = $self->{tcpquery}->sortPlayerlist($subchannel->getPlayers());
      $subchannel->{players} = \@subplayers;
    }
    $channel->{channels} = \@subchannels;
    my @players = $self->{tcpquery}->sortPlayerlist($channel->getPlayers());
    $channel->{players} = \@players;
  }
  $object->{channels} = \@channels;
  return $object;
}

# Object method: _throwException()
# Checks for errors and throws an exception
#
sub _throwException {
  my ($self, $function) = @_;
  die("Can't use function (\"".$function."\") because the virtual server is not selected") if($self->{server_udpport} != $self->{tcpquery}->{selected});
  die("Can't use function (\"".$function."\") on a stopped virtual server") if(!$self->{status});
  return 1;
}

1;